@@ -1,6 +1,14 @@
 import uproot
 import time
 import pandas
+import awkward as ak
+import os
+from tqdm import tqdm
+
+import structlog
+import logging
+log = structlog.get_logger()
+structlog.stdlib.recreate_defaults(log_level=logging.INFO)  # so we have logger names
 
 from niceplot.process_yaml import read_yaml
 
@@ -24,7 +32,7 @@ class DotAccessibleDict:
 class Reader(DotAccessibleDict):
     """Yaml Reader class for reading yaml config files, casting into useable format and prepping pandas dfs for plotting."""
     def __init__(self, config_file: str) -> None:
-        print(f"Making reader based on config file: {config_file}")
+        log.info(f"Making reader based on config file: {config_file}")
         super().__init__(self.readconfig(config_file))
         
         pass
@@ -101,8 +109,13 @@ class Reader(DotAccessibleDict):
     def prepdfdict(self) -> dict[pandas.DataFrame]:
         """Function to return dictionary with prepped pandas.DataFrames for plotting."""
 
-        # Open file with uproot:
-        susy = uproot.open(self.input_file)[self.treename]
+        # Open file(s) with uproot and store tree(s):        
+        if os.path.isdir(self.input_file):
+            # treelist = [uproot.open(os.path.join(self.input_file, filen))[self.treename] for filen in os.listdir(self.input_file)]
+            filelist = [os.path.join(self.input_file, filen) for filen in os.listdir(self.input_file)]
+        else:
+            # treelist = [uproot.open(self.input_file)[self.treename]]
+            filelist = [self.input_file]
         
         # Get list of columns to read:
         grab_columns = self.getcollist()
@@ -114,14 +127,32 @@ class Reader(DotAccessibleDict):
             if config.seedfrom is not None:
                 dfdict[config.name] = dfdict[config.seedfrom]
             else:
-                dfdict[config.name] = susy.arrays(grab_columns, library='pd')
+                # Use uproot to read into akarrays, concat & transform into pandas df:
+                akarr = ak.Array([])
+                if self.firstn: log.warning(f"Will only loop over the first {self.firstn} entires!")
+                
+                log.info()
+                log.info(f"Generating DataFrames for config {config.name}:")
+                print()
+                for filen in tqdm(filelist, desc="", unit="df"):
+
+                    with uproot.open(filen, compression=uproot.LZMA(9))[self.treename] as tree:
+                        # if firstn is provided, only run over firstn entries:
+                        if self.firstn:
+                            if len(akarr) >= self.firstn: continue
+                        
+                        # Read into akarr:
+                        akarr = ak.concatenate([akarr, tree.arrays(grab_columns, library='ak', entry_stop=self.firstn)])
+                                
+                dfdict[config.name] = ak.to_dataframe(akarr)
+                print()
             
             # Add new variables to dataframe:
             if config.newvar is not None:
                 for var in config.newvar:
                     dfdict[config.name] = self.addnewvar(dfdict[config.name], var.varname, var.formula)  
             
-            print(f"Time to read {config.name} into df: {time.time() - starttime:.2f} s") 
+            log.info(f"Total time to read config {config.name} into df: {time.time() - starttime:.2f} s") 
             
             # Make mask for constraints if masklist is provided and apply:
             if config.masklist is not None:
